// NeoACAdmin.h: interface for the NeoACAdmin class.
//
//////////////////////////////////////////////////////////////////////

#ifndef NEO_ACADMIN_H
#define NEO_ACADMIN_H

#ifdef WIN32
#pragma warning( disable : 4290 ) 
#endif

#include "API_C_dll.h"
#include "API_LEM.h"
#include "NeoConnection.h"
#include "SessionManagedNeoConnection.h"
#include "NeoException.h"


 /** @memo <DL><DD>This class is used for creating, modifying, 
   * and deleting Access Control groups, users, and rules.</DD></DL>
   * @doc <br/>This class is used for creating, modifying, 
   * and deleting Access Control groups, users, and rules.
   *
   * @author Mark Lozano
   *
   * @version 1.0
   *
   */

class HTTPCLIENT4_API NeoACAdmin  
{
	friend class NeoACAdminConnectionGuard;
public:
	// Enum used for the rule methods, values for ruleType
	enum RULE_TYPE 
	{
		RULETYPE_GROUP  = 0, 
		RULETYPE_USER   = 1
	};

	// Enum used for the rule methods, values for operation
	enum OPERATION_TYPE 
	{
		OPERATION_ALL	 = 0, 
		OPERATION_QUERY  = 1,
		OPERATION_STORE  = 2,
		OPERATION_MODIFY = 3,
		OPERATION_INSERT = 4,
		OPERATION_COPY   = 5,
		OPERATION_DELETE = 6
	};


  /** @memo <DL><DD>Creates a new instance of NeoACAdmin.</DD></DL>
	* @doc <br/>Creates a new instance of NeoACAdmin.
	*
	* @param SessionManagedNeoConnection  Instance of SessionManagedNeoConnection.
	*
	* @exception NeoException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	*/
	NeoACAdmin(SessionManagedNeoConnection* smnc)
		throw (NeoException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException);

  /** @memo <DL><DD>Creates a new instance of NeoACAdmin.</DD></DL>
	* @doc <br/>Creates a new instance of NeoACAdmin.
	*
	* @param NeoConnection  Instance of NeoConnection.
	* @param sid  The session id (SID).
	*
	* @exception NeoException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	*/

	NeoACAdmin(NeoConnection* nc, char* sid)
		throw (NeoException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException);


  /** @memo <DL><DD>Destructor for NeoACAdmin.</DD></DL>
	* @doc <br/>Destructor for NeoACAdmin.
	*
	*/
	~NeoACAdmin();

  /** @memo <DL><DD>Creates a rule that applies to a group.</DD></DL>
	* @doc <br/>Creates a rule that applies to a group. 
	*
	* @param entity  The name of the group.
	* @param operation  The operation to hide. Operation values are defined in the OPERATION_TYPE enumeration.
	* @param target  The XPath to the target node.
	* @param condition  The condition, or predicate.
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	* @exception NeoACRuleExistsException -  If the rule you are attempting to store already exists.
	*
	* @see OPERATION_TYPE
	*/
    char* createGroupRule(const char* entity, 
						  const NeoACAdmin::OPERATION_TYPE operation, 
						  const char* target, 
						  const char* condition)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException,
			   NeoACRuleExistsException);

  /** @memo <DL><DD>Creates a rule that applies to a user.</DD></DL>
	* @doc <br/>Creates a rule that applies to a user. 
	*
	* @param entity  The name of the user.
	* @param operation  The operation to hide. Operation values are defined in the OPERATION_TYPE enumeration.
	* @param target  The XPath to the target node.
	* @param condition  The condition, or predicate.
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	* @exception NeoACRuleExistsException -  If the rule you are attempting to store already exists.
	*
	* @see OPERATION_TYPE
	*/
    char* createUserRule(const char* entity, 
						 const NeoACAdmin::OPERATION_TYPE operation, 
						 const char* target, 
						 const char* condition)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException,
			   NeoACRuleExistsException);

  /** @memo <DL><DD>Deletes a rule that applies to a group.</DD></DL>
	* @doc <br/>Deletes a rule that applies to a group.
	*
	* @param groupname  The name of the group.
	* @param operation  The operation to hide. Operation values are defined in the OPERATION_TYPE enumeration.
	* @param target  The XPath to the target node.
	* @param condition  The condition, or predicate.
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	* @see OPERATION_TYPE
	*/
    char* deleteGroupRule(const char* groupname, 
						  const NeoACAdmin::OPERATION_TYPE operation,
						  const char* target, 
						  const char* condition)
		throw( NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException);

  /** @memo <DL><DD>Deletes a rule that applies to a user.</DD></DL>
	* @doc <br/>Deletes a rule that applies to a user.
	*
	* @param username  The name of the user.
	* @param operation  The operation to hide. Operation values are defined in the OPERATION_TYPE enumeration.
	* @param target  The XPath to the target node.
	* @param condition  The condition, or predicate.
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	* @see OPERATION_TYPE
	*/

    char* deleteUserRule(const char* username, 
						 const NeoACAdmin::OPERATION_TYPE operation, 
						 const char* target, 
						 const char* condition)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException);

  /** @memo <DL><DD>Create a user.</DD></DL>
	* @doc <br/>Create a user.
	*
	* @param username  The name of the user.
	* @param groupname  The group to assign this user to.
	* @param firstName  The first name for the user.
	* @param lastName  The last name for the user.
	* @param password  The password for the user. 
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	* @exception NeoACUserExistsException - If this user already exists in the database. 
	*
	*/
    char* createUser(const char* username, 
					 const char* groupname, 
					 const char* firstName, 
					 const char* lastName,
					 const char* password)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException,
			   NeoACUserExistsException);

  /** @memo <DL><DD>Delete a user.</DD></DL>
	* @doc <br/>Delete a user.
	*
	* @param username  The name of the user.
	* @param bDeleteAssociatedEntities  Specify whether or not to automatically delete the associated entities (the rules associated with this user).
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACAssociatedRulesExistException - If this user has rules applied to them and bDeleteAssociatedEntities = false.
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	*/
    char* deleteUser(const char* username, bool bDeleteAssociatedEntities = false)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACAssociatedRulesExistException,
			   NeoACException);


  /** @memo <DL><DD>Create a group.</DD></DL>
	* @doc <br/>Create a group.
	*
	* @param groupname  The name of the group.
	* @param description  The description for this group.
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	* @exception NeoACGroupExistsException - If this group already exists in the database. 
	*
	*/
    char* createGroup(const char* groupname, 
					  const char* description)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException,
			   NeoACGroupExistsException);

  /** @memo <DL><DD>Delete a group.</DD></DL>
	* @doc <br/>Delete a group.
	*
	* @param groupname  The name of the group.
	* @param bDeleteAssociatedEntities  whether or not to automatically delete the associated entities (the users assigned to this group, and the rules associated with this group). 
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	* @exception NeoACUsersExistInGroupException - If this group has users and bDeleteAssociatedEntities = false.
	* @exception NeoACAssociatedRulesExistException - If this group has rules applied to them and bDeleteAssociatedEntities = false.
	*
	*/
    char* deleteGroup(const char* groupname, bool bDeleteAssociatedEntities = false)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException,
			   NeoACUsersExistInGroupException,
			   NeoACAssociatedRulesExistException);

  /** @memo <DL><DD>Assign a user to a different group.</DD></DL>
	* @doc <br/>Assign a user to a different group.
	*
	* @param username  The name of the user.
	* @param groupname  The name of the group the user will be assigned to.
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	*/
    char* modifyUserGroupname(const char* username, 
							  const char* groupname)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException);

  /** @memo <DL><DD>Change the first name for a user. </DD></DL>
	* @doc <br/>Change the first name for a user. 
	*
	* @param username  The name of the user.
	* @param firstName  The new first name for this user. 
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	*/
    char* modifyUserFirstname(const char* username, 
							  const char* firstName)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException);

  /** @memo <DL><DD>Change the last name for a user. </DD></DL>
	* @doc <br/>Change the last name for a user. 
	*
	* @param username  The name of the user.
	* @param lastName  The new last name for this user. 
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	*/
    char* modifyUserLastname(const char* username, 
							 const char* lastName)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException);

  /** @memo <DL><DD>Change the password for a user. </DD></DL>
	* @doc <br/>Change the password for a user. 
	*
	* @param username  The name of the user.
	* @param password  The new password for this user. 
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	*/
    char* modifyUserPassword(const char* username, 
							 const char* password)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException);


  /** @memo <DL><DD>Change the description for the group.</DD></DL>
	* @doc <br/>Change the description for the group.
	*
	* @param groupname  The name of the group.
	* @param description  The new description for this group. 
	*
	* @return "&lt;success/&gt;" if successful, otherwise an Exception is thrown describing the problem. 
	*
	*
	* @exception NeoServerException - If there was an internal server error. 
	* @exception NeoCommunicationException - If there was an error communcating with the host.
	* @exception NeoInvalidArgumentException - If any parameter is null, empty, or otherwise invalid.
	* @exception NeoInvalidSessionException - If the provided session ID is not valid.
	* @exception NeoAuthorizationFailedException - If you do not have authorization to make this request. 
	* @exception NeoACException - If there was an internal Access Control exception.
	*
	*/
    char* modifyGroupDescription(const char* groupname, 
								 const char* description)
		throw (NeoServerException,
			   NeoCommunicationException,
			   NeoInvalidArgumentException,
			   NeoInvalidSessionException,
			   NeoAuthorizationFailedException,
			   NeoACException);

private:
	////////////////////
	//Member Variables
	////////////////////

    SessionManagedNeoConnection* m_smNeoConnection;
    NeoConnection* m_neoConnection;
	char m_sid[128];
	bool m_bIsSessionManaged;

	API_LEM* lem;
	

	////////////////////
	//Member Functions
	////////////////////

    NeoConnection* getConnection()
        throw (NeoInvalidArgumentException,  
               NeoPoolClosedException, 
               NeoConnectionPoolException);

    void returnConnection(NeoConnection* nc);

};

class NeoACAdminConnectionGuard
{
public:
	inline NeoACAdminConnectionGuard (NeoACAdmin* acAdmin, bool bIsSessionManaged = false) 
		: m_nc(NULL), m_acAdmin(acAdmin), entered(false), m_bIsSessionManaged(bIsSessionManaged)
	{
	}

	inline void enter() 
	{
		m_nc = m_acAdmin->getConnection();
		entered = true;
	}

	inline void exit() 
	{
		m_acAdmin->returnConnection(m_nc);
		m_nc = NULL;
		entered = false;
	}

	inline NeoConnection* getNC()
	{
		return m_nc;
	}

	inline ~NeoACAdminConnectionGuard() 
	{
		if (entered)
			exit();
	}

private:
	bool entered;
	bool m_bIsSessionManaged;
	NeoACAdmin* m_acAdmin;
	NeoConnection* m_nc;
};

#endif // NEO_ACADMIN_H

