//  Copyright 2001-2002, NeoCore Inc.
//  All Rights Reserved

#ifndef NEO_CONNECTION_H
#define NEO_CONNECTION_H
#ifdef WIN32
#pragma warning( disable : 4290 ) 
#endif

#include "NeoException.h"
#include "API_C_dll.h"



/** NeoConnection is the base object for connecting and sending commands to the
  * Neocore XMS(XML Management System).
  * It abstracts the connection process, and allows the user
  * to send XPATH type queries and commands to the XMS Server.
  * Developers should use the NeoConnection object,
  * or the SessionManagedNeoConnection object.
  * The NeoConnection class replaces the XMLProxy object, with new signatures that throw more Exceptions.
  * 
  * NOTE: If the sid is NULL and AllowAnonymousUser is set to true in the config file,
  * "anonymous user" with "guest" Access Control privileges is assumed. Only query type 
  * methods will return a response in this case, all other methods are unavailable. 
  * 
  */
class HTTPCLIENT4_API NeoConnection
{
	// This is void to hide internal structure and behavior from client apps.
	// Socket connections to the HTTP server will be pooled.
	void* url ;

public:

	// Values for getServerStatistics, types of statistics. 
	enum SERVER_STATISTIC_TYPES { ALL = 0, ADMIN, STORAGE, ACCESS, BUFFER, 
							      TRANSACTION, WINDOW };


    /** 
     * This constructor attempts to connect to the specified host and port
     * to determine if they are valid.
     * @param host The host to connect to
     * @param port The port to connect on
	 * @exception NeoMalformedURLException - If the host URL is not well-formed
     * @exception NeoCommunicationException - If NeoConnection could not be made
     * @exception NeoInvalidArgumentException - If the host is empty/null.
     */
	NeoConnection (const char* host, const int port) 
	throw (NeoMalformedURLException,
		   NeoCommunicationException, 
		   NeoInvalidArgumentException);

	/**
	 * Cleans up all alocated memory.
	 * Releases the socket connection.
	 */
	virtual ~NeoConnection (void) ;



    /**
     * Locks the entire NeoCore XMS database.
     * If you will likely lock most of the database.  It is advisable to
     * lock it ahead of time.
     *
     * @param sid Session ID
     * @param lockType Is the type of lock desired.  It must be either, SHARED, UPDATE or EXCLUSIVE
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the session id is blank or null, or if the lockType 
     * is not one of the three acceptable types.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @return XML, NeoCore XMS's response
     */
    char* globalLock(const char* sid, const char* lockType) 
    throw (NeoServerException, 
           NeoCommunicationException,
           NeoTransactionException,
           NeoInvalidArgumentException,
           NeoInvalidSessionException,           
           NeoAuthorizationFailedException);



	// Administrative commands
     /**
     * Log into the NeoCore XMS.
     *
     * @param user the user's name
     * @param password the user's passoword
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If either the user or password is invalid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request or your
     * request is denied access.
     * @return the user's SID (Session ID)
     */
	char* login(const char* user, const char* password) 
    throw  (NeoServerException,
            NeoCommunicationException,     
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoAuthorizationFailedException);


    /**
     * Logout of the NeoCore XMS
     *
     * @param sid Session ID
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML, NeoCore XMS's response.
     */
	char* logout(const char* sid) 
    throw (NeoServerException, 
           NeoCommunicationException,    
           NeoTransactionException,    
           NeoInvalidSessionException,
           NeoAuthorizationFailedException);


   /**	 
     * Determines if the SID is logged in, (i.e. a valid SID)
     *
     * @param sid Session ID
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request 
     * @return true if this connection has a session ID and that session is valid (i.e., active),
     * false otherwise.
     */
	bool isLoggedIn(const char* sid) 
    throw (NeoServerException, 
           NeoCommunicationException,
           NeoTransactionException, 
           NeoInvalidSessionException,
           NeoAuthorizationFailedException);


    /**
     * Set password on a user
     *
     * @param sid user's session ID
     * @param user the user's name
     * @param password the user's passoword
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If either the user or password is invalid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @return XML, NeoCore XMS's response.
     */
	char* setPassword(const char* sid, const char* user, const char* password) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);



    /** Sets the trace levels for the Error Log System. This method will
     * replace the current trace level setting. Previous settings are not preseved.
     * The Error Log System will maintain the values FATAL:LOG_all;ERR:LOG_all;WARNING:LOG_all
     * if they are not specified.
     *
     * @param sid the user's SID (Session ID)
     * @param tracelvl - The tracelevel to use.
     * This should be in the form: Severity:level[,level];Severity:level[,level]
     * INFO:LOG_Performance,LOG_Query is equivalent to INFO:LOG_Performance;INFO:LOG_Query
     * Set tracelvl to the empty string to revert the system trace level to
     * FATAL:LOG_all;ERR:LOG_all;WARNING:LOG_all.   
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the tracelvl is invalid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML, NeoCore XMS's response.
     */
	char* setTraceLevels(const char* sid, const char* tracelvl) 
    throw  (NeoServerException, 
            NeoCommunicationException,
            NeoTransactionException, 
            NeoInvalidArgumentException,
            NeoInvalidSessionException,             
            NeoAuthorizationFailedException);



   /** Returns the current tracelevel string being used by the Error Log.
     *
     * @param sid - The Session ID of the current Session
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @return XML, the current trace levels on NeoCore XMS.
     */
	char* getTraceLevels(const char* sid) 
    throw  (NeoServerException, 
            NeoCommunicationException,
            NeoTransactionException, 
            NeoInvalidSessionException,            
            NeoAuthorizationFailedException);



   /**
     * Activates Access Control on the NeoCore XMS.
     *
     * @param sid Session ID
     * NOTE ---> Before the XMS server can apply Access Control, there must be
     * Access Control documents stored in the server.  The defualt access control
     * document is in the NeoCore/neoxml/config directory. Before calling this method
     * store the DefaultAC.xml document in the config directory.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the supplied session id is invalid
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @return XML, NeoCore XMS's response.
     */
	char* activateAccessControl(const char* sid) 
    throw (NeoServerException, 
           NeoCommunicationException,
           NeoTransactionException, 
           NeoInvalidSessionException,
           NeoAuthorizationFailedException);


    /**
     * Store XML File
	 *
     * @param sid session ID
     * @param sourceXMLFile (required) filename of the document containing the XML to be stored
     * @param schemaURI (optional, NULL if not provided) contains the reference name of 
	 * the schema that is stored
     * @param prefixXMLFile (optional, NULL if not provided) filename of the document 
	 * containing the Meta-Data
     * of the XML being stored
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException If the source file is null, or empty.
     * @exception NeoNoFileFoundException - If any of the files cannot be located.
     * @exception NeoMalformedXMLException - If the provided XML is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @see XMLUtils#getStores
     * @see XMLUtils#getStoreLastDocID
     * @return XML success message with the document id
     */
	char* storeFileXML(const char* sid, const char *sourceXMLFile, const char *schemaURI, const char *prefixXMLFile)
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoNoFileFoundException,
            NeoMalformedXMLException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);   



    /**
     * Store XML
	 *
     * @param sid session ID
     * @param userId user id of the person making this request
     * This method will send the XML in the argument strings to the NeoCore XMS.
	 *
     * @param sourceXML XMLData containing the XML to be stored
     * @param schemaURI contains the reference name of the schema that is stored as Metadata
     * @param prefixXML XMLData containing the Meta-Data for the XML to be stored
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the source file is null, or empty.
     * @exception NeoMalformedXMLException - If the provided XML is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @see XMLUtils#getStores
     * @see XMLUtils#getStoreLastDocID
     * @return XML success message with the document id
     */
	char* storeXML(const char* sid, const char *sourceXML, const char *schemaURI, const char *prefixXML)
    throw  (NeoServerException, 
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoMalformedXMLException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);

    
	
	
    /**
     * Copy XML
     *
     * @param sid session ID
     * @param query XPath expression that uniquely identifies the document
     * to be copied
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoMalformedXPathException - If the provided XPath is not valid.
     * @exception NeoInvalidArgumentException - If the query is null, or empty.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @see XMLUtils#getCopies
     * @return XML containing the copy number of the document copied. 
     */
	char* copyXML(const char* sid, const char *query)	
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,            
            NeoMalformedXPathException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);


   
	
    /**
     * Deletes XML on the NeoCore XMS.
     * The NeoCore XMS will delete all XML fragments or documents that match the given XPath
     *
     * @param sid session ID
     * @param query an XPath expression identifying the node(s) to be deleted
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is null, or empty.
     * @exception NeoMalformedXPathException - If the provided XPath is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @see XMLUtils#getDeletions
     * @return XML containing the total number of nodes deleted.
     */
	char* deleteXML(const char* sid, const char *query) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,  
            NeoInvalidArgumentException,
            NeoMalformedXPathException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);




     /**
     * Calls the NeoCore XMS with the Query command.
     *
     * @param sid session ID
     * @param query the XPath query
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @return XML containing the query results
     */
	char* queryXML(const char* sid, const char *query) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,            
            NeoMalformedXPathException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException);

	
#ifdef CURSORING_CHUNKING
    /**
     * QueryXML with cursoring and chunking support
	 * 
     * @param query the XPath query 
     * @param profile determines the profile of your NeoQueryResult
	 * 
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoMalformedXPathException - If the provided XPath query is not valid.
     * @exception NeoCommunicationException - If there was an error communcating with the host.
     * @exception NeoTransactionException - If any transactional errors occured.
     * @exception NeoInvalidSessionException - If the managed SessionManagedNeoConnection object's SID is invalid
     * @exception NeoInvalidArgumentException - If the query argument is empty or null.
	 * @exception NeoConnectionPoolException - If this object is using a pool and there were no available connections.
	 * @exception NeoPoolClosedException - If this object is using a pool and that pool was closed.
	 * @exception NeoQueryResultExceededMaxException - If the NeoResultProfile specifies a maxResultAxisElements that
	 *            is volated by this query and they have the NeoResultProfile exceededResultsMode set to discardResults (0).
	 * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request, 
	 *            anonymous user (i.e. NULL sid)  is not allowed to execute this command	 
	 *
     * @see NeoResultProfile
     * @see NeoQueryResult
	 * @see releaseNeoQueryResult
	 *
	 * @returns a NeoQueryResult
     */
	NeoQueryResult* queryXML(const char* sid, const char *query, const NeoResultProfile* const profile) 
	throw (NeoServerException,
		   NeoMalformedXPathException,
		   NeoCommunicationException,
		   NeoTransactionException,
		   NeoInvalidSessionException,
		   NeoInvalidArgumentException,
		   NeoConnectionPoolException,
		   NeoQueryResultExceededMaxException,
		   NeoAuthorizationFailedException);					
#endif	


    /**
     * Calls the NeoCore XMS with the Query command with Update Intent meaning that the XML identified by the
     * query is locked during the query (Implicit Transaction) or until a Commit or Rollback is executed
     * (Explicit Transactions).
     *
     * @param sid session ID
     * @param query the XPath query
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML containing the query results
     */
	char* queryXMLUpdateIntent(const char* sid, const char *query) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);


#ifdef CURSORING_CHUNKING
	/**
     * QueryXMLUpdateIntent with cursoring and chunking support
     * Calls XMS with the Query command with Update Intent meaning that the XML identified by the 
     * query is locked during the query (Implicit Transaction) or until a Commit or Rollback is executed
     * (Explicit Transactions).
	 *
     * @param sid the user's SID (Session ID)
     * @param query the XPath query 
     * @param resultProfile determines the profile of your NeoQueryResult
	 *
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoMalformedXPathException - If the provided XPath query is not valid.
     * @exception NeoCommunicationException - If there was an error communcating with the host.
     * @exception NeoTransactionException - If any transactional errors occured.
     * @exception NeoInvalidSessionException - If the managed SessionManagedNeoConnection object's SID is invalid
     * @exception NeoInvalidArgumentException - If the query argument is empty or null.
	 * @exception NeoConnectionPoolException - If this object is using a pool and there were no available connections.
	 * @exception NeoPoolClosedException - If this object is using a pool and that pool was closed.
	 * @exception NeoQueryResultExceededMaxException - If the NeoResultProfile specifies a maxResultAxisElements that
	 *            is volated by this query and they have the NeoResultProfile exceededResultsMode set to discardResults (0).
	 * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request, 
	 *            anonymous user (i.e. NULL sid) is not allowed to execute this command	 
	 *
     * @see NeoResultProfile
     * @see NeoQueryResult
	 * @see releaseNeoQueryResult
	 *
	 * @returns a NeoQueryResult
	*/
	NeoQueryResult* queryXMLUpdateIntent(const char* sid, const char *query, const NeoResultProfile* const profile) 
	throw (NeoServerException,
		   NeoMalformedXPathException,
		   NeoCommunicationException,
		   NeoTransactionException,
		   NeoInvalidSessionException,
		   NeoInvalidArgumentException,
		   NeoConnectionPoolException,
		   NeoQueryResultExceededMaxException,
		   NeoAuthorizationFailedException);

#endif
    /**
     * queryFlatXML
     * Queries NeoCore XMS and returns the XML result in flattened form
     *
     * @param sid session ID
     * @param query the XPath query
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @return XML containing the query results as flattened lines
     */
	char* queryFlatXML(const char* sid, const char *query) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException);

    /**
     * queryFlatXMLUpdateIntent
     * Queries NeoCore XMS and returns the XML result in flattened form. Update Intent means 
	 * that the XML identified by the query is locked during the query (Implicit Transaction) or until 
	 * a Commit or Rollback is executed (Explicit Transactions).
     *
     *
     * @param sid session ID
     * @param query the XPath query
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML containing the query results as flattened lines
     */
	char* queryFlatXMLUpdateIntent(char* sid, char *query) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);



    /**
     * queryCountXML
     * Counts the number of nodes in the target query
     *
     * @param sid session ID
     * @param query the XPath query
     * @param acid - true causes NeoCore XMS to lock the target query during this count transaction, 
     * false indicates that this is a potential dirty-read (not acid, not locked), use false only 
     * when you are certain that this command is excecuted simultaneously with read-only transactions 
     * so that the returned data is correct.
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @see XMLUtils#getCount 
     * @return XML containing the count 
     */
	char* queryCountXML(const char* sid, const char *query, const bool acid = true) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException);


    /**
     * queryTreeXML
     * Queries for the names of the nodes one level below the target query
     *
     * @param sid session ID
     * @param query the XPath query
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the query is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @return XML containing the names of the nodes one level below the target query
     * 
     */
	char* queryTreeXML(const char* sid, const char *query) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException);


    /**
     * queryDataContextXML
     * Queries data for its context 
     *		Note: Full indexing is required on the database in order to support this command. 
     * @param sid session ID
     * @param data the data to query the context for, must be double quoted (e.g. "DataToLookFor")
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the data is blank or null
     * @exception NeoMalformedXPathException - If the data is not in the correct quoted syntax "" 
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @return XML containing metadata as a flattened line for each node containing the target query data
     */
	char* queryDataContextXML(const char* sid, const char *data) 
    throw  (NeoServerException,           
            NeoCommunicationException,           
            NeoTransactionException, 
            NeoInvalidArgumentException, 
            NeoMalformedXPathException, 
            NeoInvalidSessionException);



    /**
     * getServerStatistics
     * Gets the current NeoCore XMS Server Statistics
     *
     * @param sid session ID
     * @param statisticType the type of server statistics that your are requesting
     * valid values are:  ALL = 0, ADMIN = 1, STORAGE = 2, ACCESS = 3, BUFFER = 4, TRANSACTION = 5, WINDOW = 6
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If statisticType is invalid
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML containing the current NeoCore XMS Satistics
     */    
	char* getServerStatistics(const char* sid, const NeoConnection::SERVER_STATISTIC_TYPES statisticType) 
    throw (NeoServerException,
           NeoCommunicationException, 
           NeoTransactionException, 
           NeoInvalidArgumentException, 
           NeoInvalidSessionException,
           NeoAuthorizationFailedException);


    /**
     * clearServerStatistics
     * Clears the NeoCore XMS Server Statistics and returns the cleared statistics
     *
     * @param sid session ID
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML containing the current NeoCore XMS Satistics after they are cleared
     */
	char* clearServerStatistics(const char* sid) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);


    /**
     * getServerVersion
     * gets the NeoCore XMS Version
     *
     * @param sid session ID
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML containing the current NeoCore XMS Version 
     */
	char* getServerVersion(const char* sid)  
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);
	


   /**
     * Insert a XML fragment into a specific set of nodes on the NeoCore XMS.
     *
     * @param sid session ID
     * @param query an XPath expression identifying the beginning tag(s) after
     * which the insertion will occur
     * @param insertXML XML fragment to be inserted
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If either the query or the XML is blank or null.
     * @exception NeoMalformedXPathException - If the supplied XPath query is not valid.
     * @exception NeoMalformedXMLException - If the provided XML is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request.
     * @see XMLUtils#getInserts
     * @return XML containing the total number of nodes inserted.
     */
	char* insertXML(const char* sid, const char *query, const char *newXML) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,            
            NeoInvalidArgumentException,            
            NeoMalformedXPathException,
            NeoMalformedXMLException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);

	
    /**
     * Modifies specified node(s).
     *
     * @param sid session ID
     * @param query an XPath expression identifying the node(s) to be modified,
     * each node must be an attribute or an element
     * @param modifyXML XML fragment that will replace the text or attribute
     * value associated with the XPath-identified node
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If either the query or XML is null, or empty.
     * @exception NeoMalformedXPathException - If the provided XPath is not valid.
     * @exception NeoMalformedXMLException - If the provided XML is not valid.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @see XMLUtils#getModifications
     * @return XML containing the total number of nodes modified.
     */
	char* modifyXML(const char* sid, const char *query, const char *newXML) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoMalformedXPathException,
            NeoMalformedXMLException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);
            
 
	
		
 

	// Transaction management
    /**
     * Sends the startTransaction command to the NeoCore XMS.
     * This method will use the flush parameter in the config file
     * on the NeoCore XMS.
     *
     * This method cannot be invoked with a null or blank sid.
     * @param sid session ID
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML, NeoCore XMS's response.
     */
	char* startTransaction (const char* sid) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);






   /**
     * Sends the startTransaction command to the NeoCore XMS.
     * 
     * This method cannot be invoked with a null or blank sid.
     * @param sid session ID
     * @param flushOnCommit - If true then flush on commit, otherwise don't flush on commit
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException If there was an error communicating with the host.
     * @exception NeoTransactionException If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML, NeoCore XMS's response.
     */
	char* startTransaction (const char* sid,const bool flushOnCommit) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);

 


    /**
     * Sends the startTransaction command to the NeoCore XMS.
     * 
     * This method cannot be invoked with a null or blank sid.
	 *
     * @param sid the user's SID (Session ID)
     * @param maxDuration -  The number of seconds that are allowed between this startTransaction and
     * a subsequent commitTransaction. If this duration exceeds maxDuration then this transaction is
     * automatically rolled back. If the provided value is invalid <= 0 then the default is used.
     * @param inactivityDuration -  The number of seconds that this transaction can be inactive. 
	 * If this duration exceeds inactivityDuration then this transaction is
     * automatically rolled back. If the provided value is invalid <= 0 then the default is used.
     * @param flushOnCommit - If true then flush on commit, otherwise don't flush on commit
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML, NeoCore XMS's response.
     *									 
     */
	char* startTransaction(const char* sid,  
						   const int maxDuration, 
						   const int inactivityDuration,
						   const bool flushOnCommit) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);



    /**
     * Sends the commit transaction command to the NeoCore XMS.
     *
     * @param sid session ID
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML, NeoCore XMS's response
     */
	char* commitTransaction (const char* sid) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException, 
            NeoAuthorizationFailedException);
 

    /**
     * Sends the rollback command to the NeoCore XMS.
     *
     * @param sid session ID
     * @exception NeoServerException - If there was an internal NeoCore XMS error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML, NeoCore XMS's response.
     */
	char* rollbackTransaction (const char* sid) 
    throw  (NeoServerException,
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);



    /**
     * This sends a set isolation level command to NeoCore XMS.
     * These levels correspond to SQL2's standard.
     * READ_UNCOMMITTED - allow dirty reads.  No read locks are used. 
     * READ_COMMITTED - Read and write locks are used.  Read locks are released after each command.
     * REPEATABLE_READ - Read and write locks are used.  Read locks are not released until end of transaction.
     * No protection against phantoms.
     * The server is defaulted to REPEATABLE_READ upon the creation of a new session.
     * This is valid for the duration of the current session.
     *
     * @param level The isolation level to use from one of the acceptable levels
     * @param sid session ID
     * @exception NeoServerException - If there was an internal server error.
     * @exception NeoCommunicationException - If there was an error communicating with the host.
     * @exception NeoTransactionException - If any transactional errors occurred.
     * @exception NeoInvalidArgumentException - If the level is blank, NULL or not a valid level.
     * @exception NeoInvalidSessionException - If the provided session ID is not valid.
     * @exception NeoAuthorizationFailedException - If you do not have authorization to make this request
     * @return XML containing the isolation level after being set
     */
	char* setIsolationLevel (const char* sid, const char* level)    
    throw  (NeoServerException,  
            NeoCommunicationException,
            NeoTransactionException,
            NeoInvalidArgumentException,
            NeoInvalidSessionException,
            NeoAuthorizationFailedException);


   /**
     * Release result buffer.  All buffers returned from NeoConnection must be released. 
	 * @return void
	 */
	void releaseBuffer (char* cbuffer);



#ifdef CURSORING_CHUNKING
	/** Release NeoQueryResult All results returned must be released
	 *  
	 * @param neoQueryResult - the NeoQueryResult you want to release
	 *
	 */
	static void releaseNeoQueryResult(NeoQueryResult* neoQueryResult);
#endif



	/**
	* This is for Solaris IPlanet support only.
	* @param flag If this is set to TRUE the server is started, 
	* if FALSE the server is stopped.  
	* @param key  Key required to execute the command. 
    * @exception NeoInvalidArgumentException - If adminKey argument is empty or null.
	* @return true if successful, otherwise returns false.
	*/
	bool ControlServer(const bool flag, const char *adminKey) 
    throw (NeoInvalidArgumentException);


private:

	/** Determines if we should convert the command into a POST, if it is a query... command
     * then we compare the size of the input to the threshold, if it is a Insert or Modify then
     * we compare the input + data to the threshold. 
     * @param input - The commands input
     * @param data - The commands data.
     * @returns true if it should be converted to a post, otherwise returns false;
     */ 
     bool convertToPOST(const char* input, const char* data);


    /**
     * queryXMLInternal determines if the command should be executed as a get or a 
     * post and then executes the command appropriately
     * @param sid the user's session ID
     * @param command the command (e.g. ROLLBACK, QUERY or INSERT or DELETED or MODIFY...)
     * @param queryContext the context of the query, either /neoquery or /neoadmin
     * @param input the xpath input for the command, may be null for commands that don't
     * send input (e.g. ROLLBACK)
     * @param data the data for the command, used only for commands INSERT and MODIFY, for
     * other commands will be null
     * @return XML containing query results
     */
    char* queryXMLInternal(const char*  sid, const char* command, 
						   const char*  input = NULL, const char* data = NULL)
														throw (NeoCommunicationException,
															   NeoServerException,
															   NeoInvalidArgumentException);


	void trim(char *target);
	int isWhiteSpace(char target, char *testChars);
	void checkParam(const char *param,const char *paramName, const char *functionName, bool bCheckNullOnly = false);

};

#endif // NEO_CONNECTION_H
