//  Copyright 2001-2002, NeoCore Inc.
//  All Rights Reserved

#ifndef NEO_EXCEPTION_H
#define NEO_EXCEPTION_H

#include "string.h"

/**
 * Super class from which all NeoExceptions will inherit
 */
class NeoException
{
public:
    char msg_m[512];

	/**
	 * Gets the message associated with this exception.
	 * @return The exception message.
	 */
    inline const char* getMessage()
    {
        return msg_m;
    }

    inline NeoException(const char* msg)
    {
        strncpy(msg_m, msg, sizeof(msg_m));
    }
};

/**
 * This is thrown when the server cannot authorize against the given username and password.
 */
class NeoAuthorizationFailedException : public NeoException
{
public:
	NeoAuthorizationFailedException (const char* msg) : NeoException(msg) {}
};

/**
 * This will be thrown when a socket error occurs.  Either a connection to the
 * server is lost, or a read/write error happens.
 */
class NeoCommunicationException : public NeoException
{
public:
	NeoCommunicationException (const char* msg) : NeoException(msg) {}
};

/**
 * This is thrown if the NeoConnectionPool is used in an incorrect manner.
 * @author  Sandor Dornbush
 */
class NeoConnectionPoolException : public NeoException
{
public:
	NeoConnectionPoolException (const char* msg) : NeoException(msg) {}
};

/**
 * This is thrown if a user tries to use a method with a value that cannot work.
 * This is often thrown if parameter is blank or null.
 */
class NeoInvalidArgumentException : public NeoException
{
public:
	NeoInvalidArgumentException (const char* msg) : NeoException(msg) {}
};

/**
 * This is thrown when the session ID has expired or does not exist.
 */
class NeoInvalidSessionException : public NeoException
{
public:
	NeoInvalidSessionException (const char* msg) : NeoException(msg) {}
};

/**
 * This is thrown when a malformed XPath statement is passed in.
 */
class NeoMalformedXPathException : public NeoException
{
public:
	NeoMalformedXPathException (const char* msg) : NeoException(msg) {}
};

/**
 * This is used if a user passes in malformed XML to the XMS server.
 */
class NeoMalformedXMLException : public NeoException
{
public:
	NeoMalformedXMLException (const char* msg) : NeoException(msg) {}
} ;


/**
 * This will be thrown when a malformed URL is generated.
 */
class NeoMalformedURLException : public NeoException
{
public:
	NeoMalformedURLException (const char* msg) : NeoException(msg) {}
} ;

/**
 * This will be thrown on the Store commands when a specified file cannot be found.
 */
class NeoNoFileFoundException : public NeoException
{
public:
	NeoNoFileFoundException (const char* msg) : NeoException(msg) {}
} ;

/**
 * This is thrown if a program asks a pool for a connection after the closeAllConnections method has been called.
 */
class NeoPoolClosedException : public NeoConnectionPoolException
{
public:
	NeoPoolClosedException (const char* msg) : NeoConnectionPoolException(msg) {}
};

/**
 * This is thrown when there is an error on the server processing your request.
 */
class NeoServerException : public NeoException
{
public:
	NeoServerException (const char* msg) : NeoException(msg) {}
};

/**
 * This is thrown if the server returns any error in relation to a transaction.
 */
class NeoTransactionException : public NeoException
{
public:
	NeoTransactionException (const char* msg) : NeoException(msg) {}
};

/**
 * This is thrown if a the server hits a deadlock servicing a request.
 */
class NeoTransactionDeadlockException : public NeoTransactionException
{
public:
	NeoTransactionDeadlockException (const char* msg) : NeoTransactionException(msg) {}
};

/**
 * This is thrown if a transaction roll-back occurs on the server.
 */
class NeoTransactionRollbackException : public NeoTransactionException
{
public:
	NeoTransactionRollbackException (const char* msg) : NeoTransactionException(msg) {}
};


/**
 * This is thrown as a result of a timeout on a transaction.
 */
class NeoTransactionTimeoutException : public NeoTransactionException
{
public:
	NeoTransactionTimeoutException (const char* msg) : NeoTransactionException(msg) {}
};


/** 

  * Thrown when an unexpected error condition occurs on the NeoXMS Client 

  *

  */

class NeoClientException : public NeoException

{

public:

	NeoClientException (const char* msg) : NeoException(msg) {}

};



/** 

  * Thrown when the client destroys a NeoResultChunk and subsequently attempts to

  * cursor into or requests XML from a NeoResultChunk that is in the DESTROYED state.

  * Once NeoResultChunks are transferred from the NeoCore XMS Server to the Client they 

  * are destroyed on the Server.

  *

  * @see NeoResultChunk#NeoResultChunkState

  *

  * @author Richard A. Monteleone

  */

class NeoDestroyedResultChunkException : public NeoException

{

  public:

	  NeoDestroyedResultChunkException(const int resultChunkID, const char* msg) : NeoException(msg) 

	  {

		m_resultChunkID = resultChunkID;

	  }

   /**

     * Returns the ID of the NeoResultChunk that needs to be loaded. 

	 */

   const int get_ResultChunkID() 

   {

	   return m_resultChunkID;

   }

  private:

    /**

	  * The Result Chunk ID of the Result Chunk that was destroyed 

	  */

    int m_resultChunkID;

};





/**

  * Thrown when the NeoResultProfile resultChunkTransferMode is set to Manual and the

  * client's NeoCursor is trying to enter into a NeoResultChunk that is not in the

  * LOADED state. This exeption is thrown to inform the client that they 

  * must load the NeoResultChunk manually prior to moving or repositioning the NeoCursor

  * into or getting XML on the NeoResultChunk.

  *  

  * The client can call get_ResultChunkID on this exception to determine what

  * NeoResultChunk needs to be loaded.

  *

  * @see NeoResultProfile

  * @see NeoResultChunk#NeoResultChunkState

  * @see NeoCursor#reposition

  * @see NeoCursor#move

  * 

  *

  * @author Richard A. Monteleone

  */

class NeoManualLoadRequiredException : public NeoException

{

  public:

	  NeoManualLoadRequiredException(const int resultChunkID, const char* msg) : NeoException(msg) 

	  {

		m_resultChunkID = resultChunkID;

	  }

   /**

     * Returns the ID of the NeoResultChunk that needs to be manually loaded. 

	 */

   const int get_ResultChunkID() 

   {

	   return m_resultChunkID;

   }

  private:

    /**

      * The ID of the NeoResulChunk 

	  */

    int m_resultChunkID;

};







/** 

  * Thrown when the client attempts to move or reposition the NeoCursor outside

  * the bounary of the NeoQueryResult. If they were moving to a cursor position that is higher

  * than what exists the cursorBoundary contains the highest possible cursor position. If they

  * were moving to a cursor position that is lower than what exists cursorBoundary contains the

  * lowest possible cursor position (i.e. 0). 

  *

  * @author Richard A. Monteleone

  */

class NeoCursorOutOfRangeException : public NeoException

{

  public:



    NeoCursorOutOfRangeException(const int cursorBoundary, const char* msg): NeoException(msg) 

	{

		 m_cursorBoundary= cursorBoundary;

	}

   // Returns the boundary that was violated by the NeoCursor 

   const int get_cursorBoundary()

   {

	   return m_cursorBoundary;

   }

  private:



    // The highest Cursor Postion allowable for the QueryResult 

    int m_cursorBoundary;

};





/** 

  * Thrown when the user has specified a maximum Axis Elements size in the NeoResultProfile 

  * that is violated and has the exceededResultsMode set to discardResults (0). 

  *

  * @see NeoResultProfile

  *

  * @author Richard A. Monteleone

  */

class NeoQueryResultExceededMaxException : public NeoException

{

public:

	NeoQueryResultExceededMaxException (const char* msg) : NeoException(msg) {}

};





#endif // NEO_EXCEPTION_H
